@@ -1,8 +1,8 @@
 
 /*
- *  agInit.c  $Id: agInit.c,v 4.3 2005/01/24 00:00:30 bkorb Exp $
+ *  agInit.c  $Id: agInit.c,v 4.5 2005/07/27 17:26:31 bkorb Exp $
  *
- *  Time-stamp:      "2005-01-08 14:30:36 bkorb"
+ *  Time-stamp:      "2005-07-27 10:13:38 bkorb"
  *
  *  Do all the initialization stuff.  For daemon mode, only
  *  children will return.
@@ -24,8 +24,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 /* = = = START-STATIC-FORWARD = = = */
@@ -84,9 +84,8 @@ initialize( int arg_ct, char** arg_vec )
             SI_PLATFORM,
 #endif
             SI_MACHINE };
-        int ix;
+        int  ix;
         long sz;
-        char* pz = z+2;
 
         addSysEnv( z );
         for (ix = 0; ix < sizeof(nm)/sizeof(nm[0]); ix++) {
