@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcIf.c,v 4.3 2005/01/24 00:00:33 bkorb Exp $
+ *  $Id: funcIf.c,v 4.5 2005/07/27 17:26:32 bkorb Exp $
  *
  *  This module implements the _IF text function.
  */
@@ -21,8 +21,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 tSCC zNoIfEnd[]  = "%s ERROR:  cannot find ENDIF\n\t'%s'\n";
@@ -69,7 +69,7 @@ eval_true( void )
     char* pz = evalExpression( &needFree );
 
     if (isdigit( *pz ))
-        res = atoi( pz );
+        res = (atoi(pz) == 0) ? AG_FALSE : AG_TRUE;
 
     else switch (*pz) {
     case NUL:
